@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -35,6 +35,10 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#ifdef HAVE_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif /* HAVE_LIBNOTIFY */
+
 #include "mate-settings-manager.h"
 #include "mate-settings-profile.h"
 
@@ -43,20 +47,21 @@
 #define MATE_SESSION_DBUS_NAME      "org.mate.SessionManager"
 #define MATE_SESSION_DBUS_OBJECT    "/org/mate/SessionManager"
 #define MATE_SESSION_DBUS_INTERFACE "org.mate.SessionManager"
+#define MATE_SESSION_PRIVATE_DBUS_INTERFACE "org.mate.SessionManager.ClientPrivate"
 
-static char      *mateconf_prefix = NULL;
-static gboolean   no_daemon    = FALSE;
+/* this is kept only for compatibility with custom .desktop files */
+static gboolean   no_daemon    = TRUE;
+static gboolean   replace      = FALSE;
 static gboolean   debug        = FALSE;
 static gboolean   do_timed_exit = FALSE;
-static int        daemon_pipe_fds[2];
 static int        term_signal_pipe_fds[2];
 
 static GOptionEntry entries[] = {
-        {"debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
-        {"no-daemon", 0, 0, G_OPTION_ARG_NONE, &no_daemon, N_("Don't become a daemon"), NULL },
-        {"mateconf-prefix", 0, 0, G_OPTION_ARG_STRING, &mateconf_prefix, N_("MateConf prefix from which to load plugin settings"), NULL},
+        { "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
+        { "replace", 0, 0, G_OPTION_ARG_NONE, &replace, N_("Replace the current daemon"), NULL },
+        { "no-daemon", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &no_daemon, N_("Don't become a daemon"), NULL },
         { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time (for debugging)"), NULL },
-        {NULL}
+        { NULL }
 };
 
 static gboolean
@@ -75,6 +80,7 @@ get_bus_proxy (DBusGConnection *connection)
                                                DBUS_SERVICE_DBUS,
                                                DBUS_PATH_DBUS,
                                                DBUS_INTERFACE_DBUS);
+
         return bus_proxy;
 }
 
@@ -85,15 +91,20 @@ acquire_name_on_proxy (DBusGProxy *bus_proxy)
         guint       result;
         gboolean    res;
         gboolean    ret;
+        guint32     flags;
 
         ret = FALSE;
 
+        flags = DBUS_NAME_FLAG_DO_NOT_QUEUE|DBUS_NAME_FLAG_ALLOW_REPLACEMENT;
+        if (replace)
+            flags |= DBUS_NAME_FLAG_REPLACE_EXISTING;
+
         error = NULL;
         res = dbus_g_proxy_call (bus_proxy,
                                  "RequestName",
                                  &error,
                                  G_TYPE_STRING, MSD_DBUS_NAME,
-                                 G_TYPE_UINT, 0,
+                                 G_TYPE_UINT, flags,
                                  G_TYPE_INVALID,
                                  G_TYPE_UINT, &result,
                                  G_TYPE_INVALID);
@@ -134,6 +145,13 @@ bus_message_handler (DBusConnection *connection,
                 gtk_main_quit ();
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
+        else if (dbus_message_is_signal (message,
+                                         DBUS_INTERFACE_DBUS,
+                                         "NameLost")) {
+                g_warning ("D-Bus name lost, quitting");
+                gtk_main_quit ();
+                return DBUS_HANDLER_RESULT_HANDLED;
+        }
 
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
@@ -202,6 +220,44 @@ bus_register (DBusGConnection *bus)
 static void
 on_session_over (DBusGProxy *proxy, MateSettingsManager *manager)
 {
+        /* not used, see on_session_end instead */
+}
+
+static void
+on_session_query_end (DBusGProxy *proxy, guint flags, MateSettingsManager *manager)
+{
+        GError *error = NULL;
+        gboolean ret = FALSE;
+
+        /* send response */
+        ret = dbus_g_proxy_call (proxy, "EndSessionResponse", &error,
+                                 G_TYPE_BOOLEAN, TRUE /* ok */,
+                                 G_TYPE_STRING, NULL /* reason */,
+                                 G_TYPE_INVALID,
+                                 G_TYPE_INVALID);
+        if (!ret) {
+                g_warning ("failed to send session response: %s", error->message);
+                g_error_free (error);
+        }
+}
+
+static void
+on_session_end (DBusGProxy *proxy, guint flags, MateSettingsManager *manager)
+{
+        GError *error = NULL;
+        gboolean ret = FALSE;
+
+        /* send response */
+        ret = dbus_g_proxy_call (proxy, "EndSessionResponse", &error,
+                                 G_TYPE_BOOLEAN, TRUE /* ok */,
+                                 G_TYPE_STRING, NULL /* reason */,
+                                 G_TYPE_INVALID,
+                                 G_TYPE_INVALID);
+        if (!ret) {
+                g_warning ("failed to send session response: %s", error->message);
+                g_error_free (error);
+        }
+
         mate_settings_manager_stop (manager);
         gtk_main_quit ();
 }
@@ -258,6 +314,11 @@ static void
 set_session_over_handler (DBusGConnection *bus, MateSettingsManager *manager)
 {
         DBusGProxy *session_proxy;
+        DBusGProxy *private_proxy;
+        gchar *client_id = NULL;
+        const char *startup_id;
+        GError *error = NULL;
+        gboolean res;
 
         g_assert (bus != NULL);
 
@@ -284,6 +345,49 @@ set_session_over_handler (DBusGConnection *bus, MateSettingsManager *manager)
                                      manager,
                                      NULL);
 
+        /* Register with mate-session */
+        startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
+        if (startup_id != NULL && *startup_id != '\0') {
+                res = dbus_g_proxy_call (session_proxy,
+                                         "RegisterClient",
+                                         &error,
+                                         G_TYPE_STRING, "mate-settings-daemon",
+                                         G_TYPE_STRING, startup_id,
+                                         G_TYPE_INVALID,
+                                         DBUS_TYPE_G_OBJECT_PATH, &client_id,
+                                         G_TYPE_INVALID);
+                if (!res) {
+                        g_warning ("failed to register client '%s': %s", startup_id, error->message);
+                        g_error_free (error);
+                }
+                else {
+                        /* get org.mate.Session.ClientPrivate interface */
+                        private_proxy = dbus_g_proxy_new_for_name_owner (bus, MATE_SESSION_DBUS_NAME,
+                                                                         client_id, MATE_SESSION_PRIVATE_DBUS_INTERFACE,
+                                                                         &error);
+                        if (private_proxy == NULL) {
+                                g_warning ("DBUS error: %s", error->message);
+                                g_error_free (error);
+                        }
+                        else {
+
+                            /* get QueryEndSession */
+                            dbus_g_proxy_add_signal (private_proxy, "QueryEndSession", G_TYPE_UINT, G_TYPE_INVALID);
+                            dbus_g_proxy_connect_signal (private_proxy, "QueryEndSession",
+                                                         G_CALLBACK (on_session_query_end),
+                                                         manager, NULL);
+
+                            /* get EndSession */
+                            dbus_g_proxy_add_signal (private_proxy, "EndSession", G_TYPE_UINT, G_TYPE_INVALID);
+                            dbus_g_proxy_connect_signal (private_proxy, "EndSession",
+                                                         G_CALLBACK (on_session_end), manager, NULL);
+
+                        }
+
+                        g_free (client_id);
+                }
+        }
+
         watch_for_term_signal (manager);
         mate_settings_profile_end (NULL);
 }
@@ -306,88 +410,6 @@ msd_log_default_handler (const gchar   *log_domain,
                                unused_data);
 }
 
-
-/* We want the parent process to quit after initializing all plugins,
- * but we have to do all the work in the child process.  We can't
- * initialize in parent and then fork here: that is not clean with
- * X display and DBUS where we would make the connection from one
- * process and continue using it from the other. So, we just make the
- * parent to fork early and wait. */
-
-static void
-daemon_start (void)
-{
-        int child_pid;
-        char buf[1];
-
-        if (no_daemon)
-                return;
-
-        mate_settings_profile_msg ("forking daemon");
-
-        signal (SIGPIPE, SIG_IGN);
-        if (-1 == pipe (daemon_pipe_fds)) {
-                g_error ("Could not create pipe: %s", g_strerror (errno));
-                exit (EXIT_FAILURE);
-        }
-
-        child_pid = fork ();
-
-        switch (child_pid) {
-        case -1:
-                g_error ("Could not daemonize: %s", g_strerror (errno));
-                exit (EXIT_FAILURE);
-
-        case 0:
-                /* child */
-
-                close (daemon_pipe_fds[0]);
-
-                return;
-
-         default:
-                /* parent */
-
-                close (daemon_pipe_fds[1]);
-
-                /* Wait for child to signal that we are good to go. */
-                read (daemon_pipe_fds[0], buf, 1);
-
-                exit (EXIT_SUCCESS);
-        }
-}
-
-static void
-daemon_detach (void)
-{
-        if (no_daemon)
-                return;
-
-        mate_settings_profile_msg ("detaching daemon");
-
-        /* disconnect */
-        setsid ();
-        close (0);
-        close (1);
-        open ("/dev/null", O_RDONLY);
-        open ("/dev/null", O_WRONLY);
-
-        /* get outta the way */
-        chdir ("/");
-}
-
-static void
-daemon_terminate_parent (void)
-{
-        if (no_daemon)
-                return;
-
-        mate_settings_profile_msg ("terminating parent");
-
-        write (daemon_pipe_fds[1], "1", 1);
-        close (daemon_pipe_fds[1]);
-}
-
 static void
 parse_args (int *argc, char ***argv)
 {
@@ -416,6 +438,9 @@ parse_args (int *argc, char ***argv)
         g_option_context_free (context);
 
         mate_settings_profile_end (NULL);
+        
+        if (debug)
+            g_setenv ("G_MESSAGES_DEBUG", "all", FALSE);
 }
 
 int
@@ -441,20 +466,15 @@ main (int argc, char *argv[])
 
         parse_args (&argc, &argv);
 
-        daemon_start ();
-
         g_type_init ();
 
         mate_settings_profile_start ("opening gtk display");
         if (! gtk_init_check (NULL, NULL)) {
                 g_warning ("Unable to initialize GTK+");
-                daemon_terminate_parent ();
                 exit (EXIT_FAILURE);
         }
         mate_settings_profile_end ("opening gtk display");
 
-        daemon_detach ();
-
         g_log_set_default_handler (msd_log_default_handler, NULL);
 
         bus = get_session_bus ();
@@ -467,6 +487,10 @@ main (int argc, char *argv[])
                 goto out;
         }
 
+#ifdef HAVE_LIBNOTIFY
+        notify_init ("mate-settings-daemon");
+#endif /* HAVE_LIBNOTIFY */
+
         mate_settings_profile_start ("mate_settings_manager_new");
         manager = mate_settings_manager_new ();
         mate_settings_profile_end ("mate_settings_manager_new");
@@ -481,11 +505,7 @@ main (int argc, char *argv[])
            automatically.  Otherwise, wait for an Awake etc. */
         if (g_getenv ("DBUS_STARTER_BUS_TYPE") == NULL) {
                 error = NULL;
-                if (mateconf_prefix != NULL) {
-                        res = mate_settings_manager_start_with_settings_prefix (manager, mateconf_prefix, &error);
-                } else {
-                        res = mate_settings_manager_start (manager, &error);
-                }
+                res = mate_settings_manager_start (manager, &error);
                 if (! res) {
                         g_warning ("Unable to start: %s", error->message);
                         g_error_free (error);
@@ -493,16 +513,13 @@ main (int argc, char *argv[])
                 }
         }
 
-        daemon_terminate_parent ();
-
         if (do_timed_exit) {
-                g_timeout_add (1000 * 30, (GSourceFunc) timed_exit_cb, NULL);
+                g_timeout_add_seconds (30, (GSourceFunc) timed_exit_cb, NULL);
         }
 
         gtk_main ();
 
  out:
-        g_free (mateconf_prefix);
 
         if (bus != NULL) {
                 dbus_g_connection_unref (bus);
@@ -512,6 +529,11 @@ main (int argc, char *argv[])
                 g_object_unref (manager);
         }
 
+#ifdef HAVE_LIBNOTIFY
+        if (notify_is_initted ())
+            notify_uninit ();
+#endif /* HAVE_LIBNOTIFY */
+
         g_debug ("SettingsDaemon finished");
         mate_settings_profile_end (NULL);
 
