@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "config.h"
@@ -29,7 +29,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <mateconf/mateconf-client.h>
+#include <gio/gio.h>
 
 #include <libmatekbd/matekbd-status.h>
 #include <libmatekbd/matekbd-keyboard-drawing.h>
@@ -37,15 +37,24 @@
 #include <libmatekbd/matekbd-keyboard-config.h>
 #include <libmatekbd/matekbd-util.h>
 
-#include "msd-xmodmap.h"
 #include "msd-keyboard-xkb.h"
 #include "delayed-dialog.h"
 #include "mate-settings-profile.h"
 
 #define GTK_RESPONSE_PRINT 2
 
+#define MATEKBD_DESKTOP_SCHEMA "org.mate.peripherals-keyboard-xkb.general"
+#define MATEKBD_KBD_SCHEMA "org.mate.peripherals-keyboard-xkb.kbd"
+
+#define KNOWN_FILES_KEY "known-file-list"
+#define DISABLE_INDICATOR_KEY "disable-indicator"
+#define DUPLICATE_LEDS_KEY "duplicate-leds"
+
 static MsdKeyboardManager* manager = NULL;
 
+static GSettings* settings_desktop;
+static GSettings* settings_kbd;
+
 static XklEngine* xkl_engine;
 static XklConfigRegistry* xkl_registry = NULL;
 
@@ -57,20 +66,9 @@ static MatekbdKeyboardConfig initial_sys_kbd_config;
 
 static gboolean inited_ok = FALSE;
 
-static guint notify_desktop = 0;
-static guint notify_keyboard = 0;
-
 static PostActivationCallback pa_callback = NULL;
 static void *pa_callback_user_data = NULL;
 
-static const char KNOWN_FILES_KEY[] = "/desktop/mate/peripherals/keyboard/general/known_file_list";
-
-static const char DISABLE_INDICATOR_KEY[] = "/desktop/mate/peripherals/keyboard/general/disable_indicator";
-
-static const char DUPLICATE_LEDS_KEY[] = "/desktop/mate/peripherals/keyboard/general/duplicate_leds";
-
-static const char* mdm_keyboard_layout = NULL;
-
 static GtkStatusIcon* icon = NULL;
 
 static GHashTable* preview_dialogs = NULL;
@@ -107,6 +105,16 @@ static void msd_keyboard_log_appender(const char file[], const char function[],
 }
 #endif
 
+static void
+g_strv_behead (gchar **arr)
+{
+	if (arr == NULL || *arr == NULL)
+		return;
+
+	g_free (*arr);
+	memmove (arr, arr + 1, g_strv_length (arr) * sizeof (gchar *));
+}
+
 static void
 activation_error (void)
 {
@@ -147,7 +155,7 @@ activation_error (void)
 						      "Try using a simpler configuration or using a later version of the XFree software.")
 						     : "",
 						     "xprop -root | grep XKB",
-						     "mateconftool-2 -R /desktop/mate/peripherals/keyboard/kbd");
+						     "gsettings list-keys org.mate.peripherals-keyboard-xkb.kbd");
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
 	msd_delayed_show_dialog (dialog);
@@ -156,22 +164,18 @@ activation_error (void)
 static void
 apply_desktop_settings (void)
 {
-	MateConfClient *conf_client;
 	gboolean show_leds;
 	int i;
 	if (!inited_ok)
 		return;
 
 	msd_keyboard_manager_apply_settings (manager);
-	matekbd_desktop_config_load_from_mateconf (&current_config);
+	matekbd_desktop_config_load_from_gsettings (&current_config);
 	/* again, probably it would be nice to compare things
 	   before activating them */
 	matekbd_desktop_config_activate (&current_config);
 
-	conf_client = mateconf_client_get_default ();
-	show_leds =
-	    mateconf_client_get_bool (conf_client, DUPLICATE_LEDS_KEY, NULL);
-	g_object_unref (conf_client);
+	show_leds = g_settings_get_boolean (settings_desktop, DUPLICATE_LEDS_KEY);
 	for (i = sizeof (indicator_icons) / sizeof (indicator_icons[0]);
 	     --i >= 0;) {
 		gtk_status_icon_set_visible (indicator_icons[i],
@@ -179,14 +183,33 @@ apply_desktop_settings (void)
 	}
 }
 
+static void
+apply_desktop_settings_cb (GSettings *settings, gchar *key, gpointer   user_data)
+{
+    apply_desktop_settings ();
+}
+
 static void
 popup_menu_launch_capplet ()
 {
+	GAppInfo *info;
+	GdkAppLaunchContext *context;
 	GError *error = NULL;
 
-	gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-					  "mate-keyboard-properties",
-					  &error);
+	info = g_app_info_create_from_commandline ("mate-keyboard-properties", NULL, 0, &error);
+
+	if (info != NULL) {
+#           if GTK_CHECK_VERSION (3, 0, 0)
+		context = gdk_display_get_app_launch_context (gdk_display_get_default ());
+#           else
+		context = gdk_app_launch_context_new ();
+#           endif
+		g_app_info_launch (info, NULL,
+				   G_APP_LAUNCH_CONTEXT (context), &error);
+
+		g_object_unref (info);
+		g_object_unref (context);
+	}
 
 	if (error != NULL) {
 		g_warning
@@ -228,7 +251,11 @@ popup_menu_show_layout ()
 	    matekbd_keyboard_drawing_new_dialog (xkl_state->group,
 					      group_names
 					      [xkl_state->group]);
+#  if GTK_CHECK_VERSION(3,0,0)
+	g_signal_connect (dialog, "destroy",
+#  else
 	g_signal_connect (GTK_OBJECT (dialog), "destroy",
+#endif
 			  G_CALLBACK (show_layout_destroy),
 			  GINT_TO_POINTER (xkl_state->group));
 	g_hash_table_insert (preview_dialogs,
@@ -326,20 +353,15 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 static void
 show_hide_icon ()
 {
-	if (g_slist_length (current_kbd_config.layouts_variants) > 1) {
+	if (g_strv_length (current_kbd_config.layouts_variants) > 1) {
 		if (icon == NULL) {
-			MateConfClient *conf_client =
-			    mateconf_client_get_default ();
-			gboolean disable =
-			    mateconf_client_get_bool (conf_client,
-						   DISABLE_INDICATOR_KEY,
-						   NULL);
-			g_object_unref (conf_client);
+			gboolean disable = g_settings_get_boolean (settings_desktop, DISABLE_INDICATOR_KEY);
 			if (disable)
 				return;
 
 			xkl_debug (150, "Creating new icon\n");
 			icon = matekbd_status_new ();
+			gtk_status_icon_set_name (icon, "keyboard");
 			g_signal_connect (icon, "popup-menu",
 					  G_CALLBACK
 					  (status_icon_popup_menu_cb),
@@ -380,8 +402,7 @@ filter_xkb_config (void)
 	XklConfigItem *item;
 	gchar *lname;
 	gchar *vname;
-	GSList *lv;
-	GSList *filtered;
+	gchar **lv;
 	gboolean any_change = FALSE;
 
 	xkl_debug (100, "Filtering configuration against the registry\n");
@@ -397,27 +418,18 @@ filter_xkb_config (void)
 	}
 	lv = current_kbd_config.layouts_variants;
 	item = xkl_config_item_new ();
-	while (lv) {
-		xkl_debug (100, "Checking [%s]\n", lv->data);
-		if (matekbd_keyboard_config_split_items
-		    (lv->data, &lname, &vname)) {
+	while (*lv) {
+		xkl_debug (100, "Checking [%s]\n", *lv);
+		if (matekbd_keyboard_config_split_items (*lv, &lname, &vname)) {
+			gboolean should_be_dropped = FALSE;
 			g_snprintf (item->name, sizeof (item->name), "%s",
 				    lname);
 			if (!xkl_config_registry_find_layout
 			    (xkl_registry, item)) {
 				xkl_debug (100, "Bad layout [%s]\n",
 					   lname);
-				filtered = lv;
-				lv = lv->next;
-				g_free (filtered->data);
-				current_kbd_config.layouts_variants =
-				    g_slist_delete_link
-				    (current_kbd_config.layouts_variants,
-				     filtered);
-				any_change = TRUE;
-				continue;
-			}
-			if (vname) {
+				should_be_dropped = TRUE;
+			} else if (vname) {
 				g_snprintf (item->name,
 					    sizeof (item->name), "%s",
 					    vname);
@@ -426,20 +438,16 @@ filter_xkb_config (void)
 					xkl_debug (100,
 						   "Bad variant [%s(%s)]\n",
 						   lname, vname);
-					filtered = lv;
-					lv = lv->next;
-					g_free (filtered->data);
-					current_kbd_config.layouts_variants
-					    =
-					    g_slist_delete_link
-					    (current_kbd_config.layouts_variants,
-					     filtered);
-					any_change = TRUE;
-					continue;
+					should_be_dropped = TRUE;
 				}
 			}
+			if (should_be_dropped) {
+				g_strv_behead (lv);
+				any_change = TRUE;
+				continue;
+			}
 		}
-		lv = lv->next;
+		lv++;
 	}
 	g_object_unref (item);
 	return any_change;
@@ -448,108 +456,14 @@ filter_xkb_config (void)
 static void
 apply_xkb_settings (void)
 {
-	MateConfClient *conf_client;
 	MatekbdKeyboardConfig current_sys_kbd_config;
-	int group_to_activate = -1;
-	char *mdm_layout;
-	char *s;
 
 	if (!inited_ok)
 		return;
 
-	conf_client = mateconf_client_get_default ();
+	matekbd_keyboard_config_init (&current_sys_kbd_config, xkl_engine);
 
-	/* With MDM the user can already set a layout from the login
-	 * screen. Try to keep that setting.
-	 * We clear mdm_keyboard_layout early, so we don't risk
-	 * recursion from mateconf notification.
-	 */
-	mdm_layout = g_strdup (mdm_keyboard_layout);
-	mdm_keyboard_layout = NULL;
-
-	/* mdm's configuration and $MDM_KEYBOARD_LAYOUT separates layout and
-	 * variant with a space, but mateconf uses tabs; so convert to be robust
-	 * with both */
-	for (s = mdm_layout; s && *s; ++s) {
-		if (*s == ' ') {
-			*s = '\t';
-		}
-	}
-
-	if (mdm_layout != NULL) {
-		GSList *layouts;
-		GSList *found_node;
-		int max_groups;
-
-		max_groups =
-		    MAX (xkl_engine_get_max_num_groups (xkl_engine), 1);
-		layouts =
-		    mateconf_client_get_list (conf_client,
-					   MATEKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-					   MATECONF_VALUE_STRING, NULL);
-
-		/* Use system layouts as a default if we do not have
-		 * user configuration */
-		if (layouts == NULL) {
-			GSList *i;
-			int len;
-
-			for (i = initial_sys_kbd_config.layouts_variants;
-			     i; i = g_slist_next (i)) {
-				s = g_strdup (i->data);
-
-				/* chop off empty variants to avoid duplicates */
-				len = strlen (s);
-				if (s[len - 1] == '\t')
-					s[len - 1] = '\0';
-				layouts = g_slist_append (layouts, s);
-			}
-		}
-
-		/* Add the layout if it doesn't already exist. XKB limits the
-		 * total number of layouts. If we already have the maximum
-		 * number of layouts configured, we replace the last one. This
-		 * prevents the list from becoming full if the user has a habit
-		 * of selecting many different keyboard layouts in MDM. */
-
-		found_node =
-		    g_slist_find_custom (layouts, mdm_layout,
-					 (GCompareFunc) g_strcmp0);
-
-		if (!found_node) {
-			/* Insert at the last valid place, or at the end of
-			 * list, whichever comes first */
-			layouts =
-			    g_slist_insert (layouts, g_strdup (mdm_layout),
-					    max_groups - 1);
-			if (g_slist_length (layouts) > max_groups) {
-				GSList *last;
-				GSList *free_layouts;
-
-				last =
-				    g_slist_nth (layouts, max_groups - 1);
-				free_layouts = last->next;
-				last->next = NULL;
-
-				g_slist_foreach (free_layouts,
-						 (GFunc) g_free, NULL);
-				g_slist_free (free_layouts);
-			}
-
-			mateconf_client_set_list (conf_client,
-					       MATEKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-					       MATECONF_VALUE_STRING, layouts,
-					       NULL);
-		}
-
-		g_slist_foreach (layouts, (GFunc) g_free, NULL);
-		g_slist_free (layouts);
-	}
-
-	matekbd_keyboard_config_init (&current_sys_kbd_config,
-				   conf_client, xkl_engine);
-
-	matekbd_keyboard_config_load_from_mateconf (&current_kbd_config,
+	matekbd_keyboard_config_load_from_gsettings (&current_kbd_config,
 					      &initial_sys_kbd_config);
 
 	matekbd_keyboard_config_load_from_x_current (&current_sys_kbd_config,
@@ -572,124 +486,25 @@ apply_xkb_settings (void)
 		xkl_debug (100,
 			   "Actual KBD configuration was not changed: redundant notification\n");
 
-	if (mdm_layout != NULL) {
-		/* If there are multiple layouts,
-		 * try to find the one closest to the mdm layout
-		 */
-		GSList *l;
-		int i;
-		size_t len = strlen (mdm_layout);
-		for (i = 0, l = current_kbd_config.layouts_variants; l;
-		     i++, l = l->next) {
-			char *lv = l->data;
-			if (strncmp (lv, mdm_layout, len) == 0
-			    && (lv[len] == '\0' || lv[len] == '\t')) {
-				group_to_activate = i;
-				break;
-			}
-		}
-	}
-
-	g_free (mdm_layout);
-
-	if (group_to_activate != -1)
-		xkl_engine_lock_group (current_config.engine,
-				       group_to_activate);
 	matekbd_keyboard_config_term (&current_sys_kbd_config);
 	show_hide_icon ();
 }
 
 static void
-msd_keyboard_xkb_analyze_sysconfig (void)
+apply_xkb_settings_cb (GSettings *settings, gchar *key, gpointer   user_data)
 {
-	MateConfClient *conf_client;
-
-	if (!inited_ok)
-		return;
-
-	conf_client = mateconf_client_get_default ();
-	matekbd_keyboard_config_init (&initial_sys_kbd_config,
-				   conf_client, xkl_engine);
-	matekbd_keyboard_config_load_from_x_initial (&initial_sys_kbd_config,
-						  NULL);
-	g_object_unref (conf_client);
-}
-
-static gboolean
-msd_chk_file_list (void)
-{
-	GDir *home_dir;
-	const char *fname;
-	GSList *file_list = NULL;
-	GSList *last_login_file_list = NULL;
-	GSList *tmp = NULL;
-	GSList *tmp_l = NULL;
-	gboolean new_file_exist = FALSE;
-	MateConfClient *conf_client;
-
-	home_dir = g_dir_open (g_get_home_dir (), 0, NULL);
-	while ((fname = g_dir_read_name (home_dir)) != NULL) {
-		if (g_strrstr (fname, "modmap")) {
-			file_list =
-			    g_slist_append (file_list, g_strdup (fname));
-		}
-	}
-	g_dir_close (home_dir);
-
-	conf_client = mateconf_client_get_default ();
-
-	last_login_file_list = mateconf_client_get_list (conf_client,
-						      KNOWN_FILES_KEY,
-						      MATECONF_VALUE_STRING,
-						      NULL);
-
-	/* Compare between the two file list, currently available modmap files
-	   and the files available in the last log in */
-	tmp = file_list;
-	while (tmp != NULL) {
-		tmp_l = last_login_file_list;
-		new_file_exist = TRUE;
-		while (tmp_l != NULL) {
-			if (strcmp (tmp->data, tmp_l->data) == 0) {
-				new_file_exist = FALSE;
-				break;
-			} else {
-				tmp_l = tmp_l->next;
-			}
-		}
-		if (new_file_exist) {
-			break;
-		} else {
-			tmp = tmp->next;
-		}
-	}
-
-	if (new_file_exist) {
-		mateconf_client_set_list (conf_client,
-				       KNOWN_FILES_KEY,
-				       MATECONF_VALUE_STRING,
-				       file_list, NULL);
-	}
-
-	g_object_unref (conf_client);
-
-	g_slist_foreach (file_list, (GFunc) g_free, NULL);
-	g_slist_free (file_list);
-
-	g_slist_foreach (last_login_file_list, (GFunc) g_free, NULL);
-	g_slist_free (last_login_file_list);
-
-	return new_file_exist;
-
+    apply_xkb_settings ();
 }
 
 static void
-msd_keyboard_xkb_chk_lcl_xmm (void)
+msd_keyboard_xkb_analyze_sysconfig (void)
 {
-	if (msd_chk_file_list ()) {
-		msd_modmap_dialog_call ();
-	}
-	msd_load_modmap_files ();
+	if (!inited_ok)
+		return;
+
+	matekbd_keyboard_config_init (&initial_sys_kbd_config, xkl_engine);
+	matekbd_keyboard_config_load_from_x_initial (&initial_sys_kbd_config,
+						  NULL);
 }
 
 void
@@ -708,16 +523,6 @@ msd_keyboard_xkb_evt_filter (GdkXEvent * xev, GdkEvent * event)
 	return GDK_FILTER_CONTINUE;
 }
 
-static guint
-register_config_callback (MateConfClient * client,
-			  const char *path, MateConfClientNotifyFunc func)
-{
-	mateconf_client_add_dir (client, path, MATECONF_CLIENT_PRELOAD_ONELEVEL,
-			      NULL);
-	return mateconf_client_notify_add (client, path, func, NULL, NULL,
-					NULL);
-}
-
 /* When new Keyboard is plugged in - reload the settings */
 static void
 msd_keyboard_new_device (XklEngine * engine)
@@ -770,8 +575,7 @@ msd_keyboard_state_changed (XklEngine * engine, XklEngineStateChange type,
 }
 
 void
-msd_keyboard_xkb_init (MateConfClient * client,
-		       MsdKeyboardManager * kbd_manager)
+msd_keyboard_xkb_init (MsdKeyboardManager * kbd_manager)
 {
 	int i;
 	Display *display = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
@@ -806,31 +610,18 @@ msd_keyboard_xkb_init (MateConfClient * client,
 	if (xkl_engine) {
 		inited_ok = TRUE;
 
-		mdm_keyboard_layout = g_getenv ("MDM_KEYBOARD_LAYOUT");
+		settings_desktop = g_settings_new (MATEKBD_DESKTOP_SCHEMA);
+		settings_kbd = g_settings_new (MATEKBD_KBD_SCHEMA);
 
 		matekbd_desktop_config_init (&current_config,
-					  client, xkl_engine);
+					     xkl_engine);
 		matekbd_keyboard_config_init (&current_kbd_config,
-					   client, xkl_engine);
+					      xkl_engine);
 		xkl_engine_backup_names_prop (xkl_engine);
 		msd_keyboard_xkb_analyze_sysconfig ();
-		mate_settings_profile_start
-		    ("msd_keyboard_xkb_chk_lcl_xmm");
-		msd_keyboard_xkb_chk_lcl_xmm ();
-		mate_settings_profile_end
-		    ("msd_keyboard_xkb_chk_lcl_xmm");
 
-		notify_desktop =
-		    register_config_callback (client,
-					      MATEKBD_DESKTOP_CONFIG_DIR,
-					      (MateConfClientNotifyFunc)
-					      apply_desktop_settings);
-
-		notify_keyboard =
-		    register_config_callback (client,
-					      MATEKBD_KEYBOARD_CONFIG_DIR,
-					      (MateConfClientNotifyFunc)
-					      apply_xkb_settings);
+		g_signal_connect (settings_desktop, "changed", G_CALLBACK(apply_desktop_settings_cb), NULL);
+		g_signal_connect (settings_kbd, "changed", G_CALLBACK(apply_xkb_settings_cb), NULL);
 
 		gdk_window_add_filter (NULL, (GdkFilterFunc)
 				       msd_keyboard_xkb_evt_filter, NULL);
@@ -865,7 +656,6 @@ msd_keyboard_xkb_init (MateConfClient * client,
 void
 msd_keyboard_xkb_shutdown (void)
 {
-	MateConfClient *client;
 	int i;
 
 	pa_callback = NULL;
@@ -890,27 +680,18 @@ msd_keyboard_xkb_shutdown (void)
 	gdk_window_remove_filter (NULL, (GdkFilterFunc)
 				  msd_keyboard_xkb_evt_filter, NULL);
 
-	client = mateconf_client_get_default ();
-
-	if (notify_desktop != 0) {
-		mateconf_client_remove_dir (client, MATEKBD_DESKTOP_CONFIG_DIR,
-					 NULL);
-		mateconf_client_notify_remove (client, notify_desktop);
-		notify_desktop = 0;
+	if (settings_desktop != NULL) {
+		g_object_unref (settings_desktop);
 	}
 
-	if (notify_keyboard != 0) {
-		mateconf_client_remove_dir (client, MATEKBD_KEYBOARD_CONFIG_DIR,
-					 NULL);
-		mateconf_client_notify_remove (client, notify_keyboard);
-		notify_keyboard = 0;
+	if (settings_kbd != NULL) {
+		g_object_unref (settings_kbd);
 	}
 
 	if (xkl_registry) {
 		g_object_unref (xkl_registry);
 	}
 
-	g_object_unref (client);
 	g_object_unref (xkl_engine);
 
 	xkl_engine = NULL;
